package org.processmining.featuremerge.dialogs;


import java.awt.BorderLayout;
import java.text.NumberFormat;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumnModel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.featuremerge.confs.Configuration;


public class AttributesSelectorDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3587595452197538653L;
	@SuppressWarnings("unused")
	
	JTable table;
	
	public AttributesSelectorDialog(UIPluginContext context, Configuration conf) {
		Configuration configuration=conf;
		
		JLabel introduction = new JLabel("<html>"
				+ "Choose the numeric attributes you want to use to derive a new attribute."
				+ "</html>");
		introduction.setAlignmentX(LEFT_ALIGNMENT);
		add(introduction);
		
		final int numRows=conf.getKPIs().size();
		final String[] columnsNames= {"Select","Attribute name","Min value","Max value","Avg value", "Std dev"};
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		table = new JTable();

		DefaultTableModel model=new DefaultTableModel(){
		      public Class<?> getColumnClass(int column)
		      {
		        switch(column)
		        {
		        case 0:
		          return Boolean.class;
		        case 1:
		          return String.class;
		        case 2:
		          return String.class;
		        case 3:
		          return String.class;
		        case 4:
		        	return String.class;
		        case 5:
		        	return String.class;
		        default:
		            return String.class;
		        }
		      }
		    };
		    table.setModel(model);
		    
		// set titles
		for(int i=0;i<columnsNames.length;i++)
			model.addColumn(columnsNames[i]);
	
		//shrink the first column
		TableColumnModel tcm=table.getColumnModel();
	    tcm.getColumn(0).setPreferredWidth(40);
	  
		//set all values
		int i=0;
		for(String kpi:conf.getKPIs().getKPIs()) {
			model.addRow(new Object[0]);
			model.setValueAt(false, i, 0);
			model.setValueAt(kpi, i, 1);
			model.setValueAt(reduceDecimalDigits(conf.getKPIs().getMin(kpi),2), i, 2);
			model.setValueAt(reduceDecimalDigits(conf.getKPIs().getMax(kpi),2), i, 3);
			model.setValueAt(reduceDecimalDigits(conf.getKPIs().getAvg(kpi),2), i, 4);
			model.setValueAt(reduceDecimalDigits(conf.getKPIs().getStdv(kpi),2), i, 5);
			i++;
		}
		
		DefaultTableCellRenderer centerRenderer = new DefaultTableCellRenderer();
		centerRenderer.setHorizontalAlignment(SwingConstants.CENTER);
		table.setDefaultRenderer(String.class, centerRenderer);
		
			JScrollPane tableContainer = new JScrollPane(table);
			add(tableContainer, BorderLayout.CENTER);
		//add(table,BorderLayout.CENTER);
	}
	
	public String[] getSelectedRows() {
		ArrayList<String> selected=new ArrayList<String>(); 
		for (int i = 0; i < table.getRowCount(); i++) {
		     Boolean isChecked = Boolean.valueOf(table.getValueAt(i,0).toString());
		     if (isChecked) {
		       selected.add(table.getValueAt(i,1).toString());
		    } 
		}
		return selected.toArray(new String[0]);
	}
	
	/**
	 * Takes a double and reduce the number of its decimal digits to the specified number
	 * @param value
	 * @param num
	 * @return
	 */
	private double reduceDecimalDigits(double value, int num) {
		NumberFormat nf = NumberFormat.getInstance(); // get instance
		nf.setMaximumFractionDigits(num); // set decimal places
		String stringva=nf.format(value);
		stringva=stringva.replace(".","").replace(",",".");
		System.out.println(value+" > "+stringva);
		return Double.valueOf(stringva);
		
	}
	
	
}