/******************************************************************************
 * Compilation: javac Histogram.java Execution: java Histogram n trials
 *
 * This data type supports simple client code to create dynamic histograms of
 * the frequency of occurrence of values in [0, N). The frequencies are kept in
 * an instance-variable array, and an instance variable max tracks the maximum
 * frequency (for scaling).
 *
 * % java Histogram 50 1000000
 *
 ******************************************************************************/

package org.processmining.featuremerge.dialogs;

import java.util.ArrayList;

import javax.swing.JDialog;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.statistics.HistogramDataset;
import org.jfree.data.statistics.HistogramType;
import org.processmining.featuremerge.confs.Configuration;

public class HistogramDialog extends JDialog {

	/**
	* 
	*/
	private static final long serialVersionUID = 1L;

	public HistogramDialog(Configuration conf,String kpi) {
	
		ArrayList<Double> valuesArray = conf.getKPIs().getRow(kpi);
		double[] values=new double[valuesArray.size()];
		for (int i = 0; i < valuesArray.size(); i++) {
		    values[i] = valuesArray.get(i);                // java 1.5+ style (outboxing)
		 }
		//get the number of different values
		int numberOfValues=findNumOfValues(values);
		int bins;
		//define the number of bins
		if(numberOfValues<10)
			bins=numberOfValues;
		else if(numberOfValues<50)
			bins=20;
		else if(numberOfValues<100)
			bins=40;
		else bins=100;
		
		HistogramDataset dataset = new HistogramDataset();
		dataset.setType(HistogramType.RELATIVE_FREQUENCY);
		dataset.addSeries("Histogram", values, bins);
		String plotTitle = "Frequency distribution";
		PlotOrientation orientation = PlotOrientation.VERTICAL;
		boolean show = false;
		boolean toolTips = false;
		boolean urls = false;
		JFreeChart chart = ChartFactory.createHistogram(plotTitle, "", "", dataset, orientation, show, toolTips, urls);
		ChartPanel chartPanel = new ChartPanel(chart);
		add(chartPanel);
	}
	
	/**
	 * Extract the number of distinct values in the distribution for a certain indicator
	 * @param values
	 * @return an int
	 */
	private int findNumOfValues(double[] values) {
		ArrayList<Double> numbers=new ArrayList<Double>();
		for(int i=0;i<values.length;i++) {
			if(!numbers.contains(values[i]))
				numbers.add(values[i]);
		}
		return numbers.size();
	}
}
