package org.processmining.featuremerge.dialogs;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.NumberFormatter;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.featuremerge.confs.Configuration;
import org.processmining.featuremerge.models.Threshold;
import org.processmining.featuremerge.plugin.DistributionCalculator;
import org.processmining.framework.util.ui.widgets.ProMPropertiesExpertModePanel;
import org.processmining.framework.util.ui.widgets.WidgetColors;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ThresholdSelectorDialog extends JPanel {
	
	private ProMPropertiesExpertModePanel panel;
	private JLabel coveringLabel2;
	private Configuration conf;
	private ArrayList<SliderValues> sliders;
	
	public ThresholdSelectorDialog(UIPluginContext context,Configuration configuration) {
		this.conf=configuration;
		
		//ArrayList<Threshold> thresholds=new ArrayList<Threshold>();
		sliders=new ArrayList<SliderValues>();

		setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		JLabel introduction = new JLabel("<html>"
				+ "For each selected attribute, select a condition and a threshold.<br/>All traces satisying the set of constraints will have the new attribute set to 1."
				+ "</html>");
		//introduction.setAlignmentX(LEFT_ALIGNMENT);
		add(introduction);
		
		
		SlickerFactory sl=SlickerFactory.instance();
		
		JPanel bigPanel=sl.createRoundedPanel();
		bigPanel.setPreferredSize(new Dimension(600, 400));
		bigPanel.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		bigPanel.setLayout(new BoxLayout(bigPanel, BoxLayout.Y_AXIS));
		
		// for each selected attribute
		for(String kpi:conf.getSelected()) {
			
			int min=(int)Math.round(conf.getKPIs().getMin(kpi));
			int max=(int)Math.round(conf.getKPIs().getMax(kpi));
			
			//Create the label.
	        JLabel sliderLabel = sl.createLabel(kpi);//, JLabel.CENTER);
	      //  sliderLabel.setAlignmentX(CENTER_ALIGNMENT);
	        
	        //Create the formatted text field and its formatter.
	        NumberFormat numberFormat = NumberFormat.getNumberInstance();
	        NumberFormatter formatter = new NumberFormatter(numberFormat);
	        numberFormat.setGroupingUsed(false);
	        formatter.setMinimum(min);
	        formatter.setMaximum(max);
	        final JFormattedTextField textField = new JFormattedTextField(formatter);
	        textField.setValue((int)Math.round(conf.getKPIs().getAvg(kpi)));
	        textField.setColumns(5); //get some space
			textField.setEditable(true);
			textField.setMaximumSize(new Dimension(70,20));
			
			
			// define a JSlider
			final JSlider slider = sl.createSlider(JSlider.HORIZONTAL);//,min,max, Math.round(max-min)/2);
			slider.setMaximum(max*10);
			slider.setMinimum(min);
			slider.setValue(Integer.parseInt(textField.getValue().toString()));
			/**if((max-min)>10) {
			//slider.setMinorTickSpacing(Math.round((max-min)/10));
				slider.setMajorTickSpacing(Math.round((max-min)/10));
				slider.setPaintTicks(true);
				slider.setPaintLabels(true);
			}**/
			slider.setMaximumSize(new Dimension(400,30));
			slider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					int value=slider.getValue();
					textField.setValue((double)(value)/10);
				}});
			
			textField.addKeyListener(new KeyAdapter(){
	            public void keyReleased(KeyEvent ke) {
	                String typed = textField.getText();
	                if(!typed.matches("\\d+(\\,\\d)?")) {
	                    return;
	                }
	                slider.setValue((int)Double.parseDouble(typed)*10);
	            }
	        });
			
			//create the comboBox
			String[] comboOptions = { "<","<=", ">",">="};
			JComboBox combo = sl.createComboBox(comboOptions);
			combo.setPreferredSize(new Dimension(50,30));
			
			//create the button for the distribution plot
			Icon icon=new ImageIcon("/home/es/eclipse-workspace/ProM/icon.gif");
			final JButton buttonIcon=sl.createButton(">");
			buttonIcon.setIcon(icon);
			buttonIcon.setName(kpi);
			
			//buttonIcon.setMargin(new Insets(0, 0, 0, 0));
			//buttonIcon.setBorder(null);
			buttonIcon.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // update the value of the covering number
				  JDialog dialog=new HistogramDialog(conf, buttonIcon.getName());
				  dialog.setSize(350,350);
				  dialog.setPreferredSize(new Dimension(350,350));
				  dialog.setLocationRelativeTo(null);
				 //dialog.setUndecorated(true);
				  dialog.pack();
				  dialog.setVisible(true);
			  }
			});
			
			//create a JPanel for the label and slider 
			 JPanel kpiPanel = new JPanel(); //use FlowLayout
			 kpiPanel.setLayout(new BoxLayout(kpiPanel, BoxLayout.X_AXIS));
		     kpiPanel.add(sliderLabel);
		     kpiPanel.add(combo);
		     kpiPanel.add(textField);
		     kpiPanel.add(slider);
		     kpiPanel.add(buttonIcon);
		 	 kpiPanel.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
			 kpiPanel.setAlignmentX(LEFT_ALIGNMENT);
			 kpiPanel.setPreferredSize(new Dimension(600,50));
		     //add the slider/combo/kpi to the sliders array to reference
		     SliderValues item=new SliderValues(kpi,combo,slider);
		     sliders.add(item);
		     bigPanel.add(kpiPanel);
		     //this.add(Box.createVerticalGlue());
		}
		add(bigPanel);
		
		/**JPanel fakePanel=new JPanel();
		fakePanel.setPreferredSize(new Dimension(600,100));
		add(fakePanel);**/
		
		double coveringValue=DistributionCalculator.calculateCovering(conf, sliders);
		JPanel coveringPanel=new JPanel();
		coveringPanel.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		coveringPanel.setAlignmentX(LEFT_ALIGNMENT);
		coveringPanel.setPreferredSize(new Dimension(600,30));
		coveringPanel.setLayout(new BoxLayout(coveringPanel,BoxLayout.X_AXIS));
		JLabel coveringLabel=sl.createLabel("% of traces selected: ");
		coveringLabel2=sl.createLabel(String.valueOf(coveringValue));
		coveringPanel.add(coveringLabel);
		coveringPanel.add(coveringLabel2);
		JButton button=sl.createButton("Update");
		button.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			    // update the value of the covering number
				  double coveringValue=DistributionCalculator.calculateCovering(conf,sliders);
				  coveringLabel2.setText(String.valueOf(coveringValue));
			  }
			});
		coveringPanel.add(button);
		add(coveringPanel);
	}
	/**
	 * Returns an HashMap linking each selected indicator to the corresponding constraint (in terms of type of comparison operator and threshold)
	 * @return an HashMap
	 */
	public HashMap<String,Threshold> getThresholds() {
		HashMap<String,Threshold> hm=new HashMap<String,Threshold>();
		
		//extract from sliders an hashmap
		for(SliderValues s:sliders) {
			Threshold threshold=new Threshold(s.getComparionType(),s.getSliderValue());
			hm.put(s.getKpi(),threshold);
		}
		return hm;
		
	}
	
	public class SliderValues{
		String kpi;
		JComboBox box;
		JSlider slider;
		
		public SliderValues(String kpi,JComboBox box,JSlider slider) {
			this.kpi=kpi;
			this.box=box;
			this.slider=slider;
		}
		
		public String getKpi() {
			return kpi;
		}
		
		public JComboBox getJComboBox() {
			return box;
		}
		
		public JSlider getJSlider() {
			return slider;
		}
		
		public double getSliderValue() {
			return (double)(slider.getValue()/10);
		}
		
		public String getComparionType() {
			return String.valueOf(box.getSelectedItem());
		}
		
	}
}
