package org.processmining.featuremerge.plugin;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.processmining.featuremerge.confs.Configuration;
import org.processmining.featuremerge.dialogs.ThresholdSelectorDialog.SliderValues;

public class DistributionCalculator {

		private static HashMap<String,ArrayList<Integer>> results;
	
		public static double calculateCovering(Configuration conf,ArrayList<SliderValues> sliders) {
			results=new HashMap<String,ArrayList<Integer>>();
			//for each selected attribute
			for(int j=0;j<sliders.size();j++) {
				ArrayList<Integer> targetIndexes=new ArrayList<Integer>();
				SliderValues s=sliders.get(j);
				String kpi=s.getKpi();
				ArrayList<Double> values=conf.getKPIs().getRow(kpi);
				// according to the selected comparison type check which values satisfy the conditions 
				String comparisonType=s.getComparionType();
				double threshold=s.getSliderValue();
				
				switch(comparisonType) {
					case(">"):
						for(int i=0;i<values.size();i++) {
							if(values.get(i)>threshold)
								targetIndexes.add(i);
						}
						break;
					case(">="):
						for(int i=0;i<values.size();i++) {
							if(values.get(i)>=threshold)
								targetIndexes.add(i);
						}
						break;
					case("<"):
						for(int i=0;i<values.size();i++) {
							if(values.get(i)<threshold)
								targetIndexes.add(i);
						}
						break;
					case("<="):
						for(int i=0;i<values.size();i++) {
							if(values.get(i)<=threshold)
								targetIndexes.add(i);
						}
						break;
						default: break;
				}
				//in results there are, for each kpi, the list of indexes of traces which satisfy the i-th condition
				results.put(kpi, targetIndexes);
			}
			return ((double)evaluateCovering()/(double)conf.getKPIs().getNumValuesInRow());
		}
		/** 
		 * The function evaluate the number of traces that satisfy all the conditions
		 * @return the number of traces
		 */
		private static int evaluateCovering() {
			//order names to iterate on them
			Object[] namesO=results.keySet().toArray();
			String[] names = Arrays.copyOf(namesO, namesO.length, String[].class);
			//initialize as partialResult the values of the first indicator 
			ArrayList<Integer> partialResult=results.get(names[0]);
			//cycle through all indicators intesecting the current set with the values of the next one
			for(int i=0;i<names.length;i++) {
				if(results.get(names[i]).size()>0) {
					partialResult=(ArrayList<Integer>)intersection(partialResult,results.get(names[i]));
				}
				else return 0;
			}
			//return the cardinality the of the intersection
			return partialResult.size();
		}
		
		 private static <T> List<T> intersection(List<T> list1, List<T> list2) {
		        List<T> list = new ArrayList<T>();

		        for (T t : list1) {
		            if(list2.contains(t)) {
		                list.add(t);
		            }
		        }

		        return list;
		    }
		 /**
		  * Check whether the i-th trace satisfy the constraints put on the new attribute
		  * @param conf
		  * @param trace
		  * @return a boolean
		  */
		 public static long checkContraintSatisfaction(Configuration conf, XEvent event) {
			 int outcome=0;
			 for(String kpi:conf.getThresholds().keySet()) {
				 XAttributeMap xAttrMap=event.getAttributes();
				 //get the attribute value
				 XAttribute xAttr=xAttrMap.get(kpi);
				 double value=Double.parseDouble(xAttr.toString());
				 double threshold=conf.getThresholds().get(kpi).getValue();
				 String comparison=conf.getThresholds().get(kpi).getComparison();
				 switch(comparison) {
					 case(">"):
						 if((value>threshold))
							 outcome++;
					 	 break;
					 case(">="):
						 if((value>=threshold))
							 outcome++;
					 	 break;
					 case("<"):
						 if((value<threshold))
							 outcome++;
					 	 break;
					 case("<="):
						 if((value<=threshold))
							 outcome++;
					 	 break;		
					 default:
				 }
			 }
			 return outcome;
		 }
}
