package org.processmining.kpiprocessenhancementworkflow.confs;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.kpiprocessenhancementworkflow.models.DatasetColumn;
import org.processmining.kpiprocessenhancementworkflow.models.DatasetEntry;
import org.processmining.kpiprocessenhancementworkflow.models.PrefixContainer;
import org.processmining.kpiprocessenhancementworkflow.models.RuleSet;

import weka.core.Instance;


public class Configuration {

	private PrefixContainer pc;
	private PrefixContainer pc_model;
	private HashMap<String,DatasetEntry> table;
	private HashMap<DatasetColumn,Integer> columns;
	private Set<DatasetColumn> selectedColumns;
	private HashMap<String,Integer> selectedGenericColumns;
	private ArrayList<String> selectedActivities;
	private ArrayList<RuleSet> rulesets;
	private HashMap<String,ArrayList<String>> alignmentNewNameMap;
	private HashMap<RuleSet, ArrayList<RuleSet>> mapTreeRuleSetWithAprioriRuleset;
	private ArrayList<Instance> instanceSet;
	
	private DataPetriNet net;
	private String indicator;
	private Set<Object> indicatorValues;
	private int threshold;
	private HashMap<ArrayList<String>,String> expressions;

	//Decision tree
	private HashMap<String,Object> decisionTreeConfiguration=new HashMap<String,Object>();
	
	
	public Configuration() {
		indicatorValues=new HashSet<Object>();
		expressions=new HashMap<ArrayList<String>,String>();
		rulesets=new ArrayList<RuleSet>();
		alignmentNewNameMap=new HashMap<String,ArrayList<String>>();
		mapTreeRuleSetWithAprioriRuleset=new HashMap<RuleSet, ArrayList<RuleSet>>();	
		instanceSet= new ArrayList<Instance>();
	}
	
	public int getThreshold() {
		return threshold;
	}

	public void setThreshold(int threshold) {
		this.threshold = threshold;
	}

	
	public Set<Object> getIndicatorValues() {
		return indicatorValues;
	}

	public void setIndicatorValues(Set<Object> indicatorValues) {
		this.indicatorValues = indicatorValues;
	}
	
	public void addIndicatorValue(Object value) {
		indicatorValues.add(value);
	}
	
	public void setIndicator(String indicator) {
		this.indicator = indicator;
	}

	public void setTable(HashMap<String,DatasetEntry> table) {
		this.table=table;
	}
	
	public void resetTable() {
		this.table=new HashMap<String,DatasetEntry>();
		
	}
	public void setColumns(HashMap<DatasetColumn,Integer> columns) {
		this.columns=columns;
	}
		
	public void setPrefixContainer(PrefixContainer pc, boolean isModel) {
		if(!isModel)
			this.pc=pc;
		else
			this.pc_model=pc;
	}

	public String getIndicator() {
		return indicator;
	}
	
	public PrefixContainer getPc(boolean isModelMove) {
		if(!isModelMove)
			return pc;
		else 
			return pc_model; 
	}
	
	
	public HashMap<String,DatasetEntry> getTable(){
		return table;
	}
	
	public HashMap<DatasetColumn,Integer> getColumns(){
		return columns;
	}

	public void addActivityToPc(String activity,boolean isModelMove) {
		if(!isModelMove)
			pc.addActivity(activity);
		else
			pc_model.addActivity(activity);
	}

	public void addPrefixToActivityPc(String activity, Set<String> currentPrefix, boolean isModelMove) {
		if(!isModelMove)
			pc.addPrefixToActivity(activity, currentPrefix);
		else
			pc_model.addPrefixToActivity(activity, currentPrefix);
	}

	public void increaseFrequencyPc(String activity, Set<String> currentPrefix, boolean isModelMove) {
		if(!isModelMove)
			pc.increaseFrequency(activity, currentPrefix);
		else
			pc_model.increaseFrequency(activity, currentPrefix);
		
	}

	public void replaceColumn(DatasetColumn dd, Integer integer) {
		columns.replace(dd,integer);
		
	}

	public void putColumns(DatasetColumn column, int i) {
		columns.put(column, i);
		
	}

	public void putTable(String trace, DatasetEntry entry) {
		table.put(trace, entry);
		
	}

	public void addExpression(ArrayList<String> exprs, String classValue) {
		expressions.put(exprs, classValue);
	}
	
	public HashMap<ArrayList<String>,String> getExpressions() {
		return expressions;
	}

	public void setSelectedColumns(Set<DatasetColumn> selectedColumns) {
		this.selectedColumns=selectedColumns;
	}
	
	public Set<DatasetColumn> getSelectedColumns(){
		return selectedColumns;
	}

	public void setSelectedGenericColumns(HashMap<String, Integer> selectedGenericColumns) {
		this.selectedGenericColumns=selectedGenericColumns;
	}
	
	public HashMap<String,Integer> getSelectedGenericColumns(){
		return selectedGenericColumns;
	}

	public void setSelectedActivities(ArrayList<String> selectedActivities) {
		this.selectedActivities=selectedActivities;
	}
	
	public ArrayList<String> getSelectedActivities(){
		return selectedActivities;
	}

	public void setNet(DataPetriNet net) {
		this.net=net;
		
	}
	
	public DataPetriNet getNet() {
		return net;
	}

	public void addRuleSet(RuleSet ruleset) {
		rulesets.add(ruleset);
	}
	
	public ArrayList<RuleSet> getRulesets(){
		return rulesets;
	}
	
	public void createNewAlignmentNewName(String trace_id) {
		alignmentNewNameMap.put(trace_id, new ArrayList<String>());
	}
	
	public void resetAlignmentNewName() {
		alignmentNewNameMap=new HashMap<String,ArrayList<String>>();
	}
	
	public void putAlignmentNewName(String trace_id,String newname) {
		if(!alignmentNewNameMap.keySet().contains(trace_id))
			createNewAlignmentNewName(trace_id);
		ArrayList<String> newnames=alignmentNewNameMap.get(trace_id);
		newnames.add(newname);
		alignmentNewNameMap.put(trace_id, newnames);
	}
	
	/**
	 * Returns the variant name (i.e. for the specific step of the specified trace_id)
	 * @param trace_id
	 * @param stepNum
	 * @return
	 */
	public String getNewName(String trace_id,int stepNum) {
		String output="";
		ArrayList<String> arr=alignmentNewNameMap.get(trace_id);
		try {
			output=arr.get(stepNum);
		}
		catch(Exception e) {
			System.out.println("exception");
			e.printStackTrace();
		}
		return output;
	}

	/**
	 * Maps a J48 rulesets (related to a branch leading to a cluster with high KPI) to a set of rulesets obtined from the Apriori algorithm
	 * @param ruleset
	 * @param newRules
	 */
	public void mapTreeRuleSetWithAprioriRuleset(RuleSet ruleset, ArrayList<RuleSet> newRules) {
		if(mapTreeRuleSetWithAprioriRuleset.containsKey(ruleset))
			mapTreeRuleSetWithAprioriRuleset.get(ruleset).addAll(newRules);
		else {
			ArrayList<RuleSet> newArr=new ArrayList<RuleSet>();
			newArr.addAll(newRules);
			mapTreeRuleSetWithAprioriRuleset.put(ruleset, newArr);
		}
	}
	
	/**
	 * Return the set of rulesets obtained from the Apriori algorithm that are linked to the input ruleset generated by the J48
	 * @param J48Ruleset
	 * @return
	 */
	public ArrayList<RuleSet> getMappedAprioriRulesets(RuleSet J48Ruleset){
		if(mapTreeRuleSetWithAprioriRuleset.containsKey(J48Ruleset))
			return mapTreeRuleSetWithAprioriRuleset.get(J48Ruleset);
		else return null;
	}
	
	public HashMap<String,Object> getDecisionTreeConfigurationItem() {
		return decisionTreeConfiguration;
	}
	
	public void putDecisionTreeConfigurationItem(String item,Object value){
		decisionTreeConfiguration.put(item,value);
	}
	
	public void deleteDecisionTreeConfiguration() {
		decisionTreeConfiguration=new HashMap<String,Object>();
	}

	/**
	 * Adds one instance for the Classification
	 * @param instance
	 */
	public void addInstance(Instance instance) {
		instanceSet.add(instance);
		
	}

	/**
	 * Returns all instances for the Classification
	 * @return
	 */
	public ArrayList<Instance> getAllInstances(){
		return instanceSet;
	}

}
