package org.processmining.kpiprocessenhancementworkflow.dialogs;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class J48ConfigurationPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final JCheckBox pruneBox;
	private final NiceDoubleSlider confThreshold;
	private final NiceIntegerSlider minNumInstancePerLeaf;
	private final NiceIntegerSlider numFoldErrorPruning;
	private final NiceDoubleSlider aprioriThreshold;
	private final NiceDoubleSlider aprioriDeltaThreshold;
	private final JCheckBox binaryBox;
	private final JCheckBox aprioriBox;
	
	public J48ConfigurationPanel(int logSize, int numVariants)
	{
		SlickerFactory instance=SlickerFactory.instance();
		
		JLabel message=instance.createLabel(numVariants+" attributes have been selected and will be used for the training of the classificator. Please configure J48 parameters.");
		pruneBox= instance.createCheckBox("Prune Tree", true);
		binaryBox= instance.createCheckBox("Binary Tree", false);
		aprioriBox=instance.createCheckBox("Execute itemset mining to find more rules (beta)", false);
		aprioriThreshold=instance.createNiceDoubleSlider("Set the support threshold for the itemset mining", 0, 1, 0.95, Orientation.HORIZONTAL);
		aprioriDeltaThreshold=instance.createNiceDoubleSlider("Set the delta threshold for the itemset mining", 0, 1, 0.01, Orientation.HORIZONTAL);
		confThreshold = instance.createNiceDoubleSlider("Set confidence threshold for pruning", 0.1, 1, 0.25, Orientation.HORIZONTAL);
		minNumInstancePerLeaf= instance.createNiceIntegerSlider("Minimum Number of instances per leaf ", 2, logSize, 2, Orientation.HORIZONTAL);
		numFoldErrorPruning= instance.createNiceIntegerSlider("Number of folds for reduced error pruning", 2, 10, 3, Orientation.HORIZONTAL);
		

		setLayout(new GridLayout(11,1));
		add(message);
		add(pruneBox);
		add(binaryBox);
		add(confThreshold);
		add(minNumInstancePerLeaf);
		add(numFoldErrorPruning);
		aprioriThreshold.enable(false);
		aprioriBox.addActionListener(new ActionListener()
		{
			  public void actionPerformed(ActionEvent e)
			  {
			      if(aprioriBox.isSelected())
					  aprioriThreshold.enable(true);
				  else
					  aprioriThreshold.enable(false);
			  }
			});
		add(aprioriBox);
		add(aprioriThreshold);
		add(aprioriDeltaThreshold);
		
	}
	
	public boolean prunedTree() {
		return pruneBox.isSelected();
	}

	public boolean binaryTree() {
		return binaryBox.isSelected();
	}
	
	public boolean apriori() {
		return aprioriBox.isSelected();
	}
	
	public float getConfidenceThreshold() {
		return (float) confThreshold.getValue();
	}

	public int getMinNumInstancePerLeaf() {
		return minNumInstancePerLeaf.getValue();
	}

	public int getNumFoldErrorPruning() {
		return numFoldErrorPruning.getValue();
	}

	public Double aprioriThreshold() {
		return aprioriThreshold.getValue();
	}

	public Double aprioriDeltaThreshold() {
		return aprioriDeltaThreshold.getValue();
	}
}
