package org.processmining.kpiprocessenhancementworkflow.dialogs;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.ToolTipManager;

import org.processmining.kpiprocessenhancementworkflow.confs.Configuration;
import org.processmining.kpiprocessenhancementworkflow.models.DatasetColumn;
import org.processmining.kpiprocessenhancementworkflow.models.PrefixContainer.PrefixFrequencyPair;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class VariantSelector extends JDialog {

	private ArrayList<JCheckBox> variantCheckBoxes;
	private JCheckBox genericCheckBox;
	private String generic;
	private HashMap<String,ArrayList<String>> selections;
	private ArrayList<String> allVariants;
	
	public VariantSelector(final String generic, final HashMap<String,ArrayList<String>> selections,Configuration conf) {
		ArrayList<PrefixFrequencyPair> arrPrefix;
		if(generic.contains("Log move "))
			arrPrefix=conf.getPc(false).getPrefixes(generic.replace("Log move ",""));
		else
			arrPrefix=conf.getPc(true).getPrefixes(generic.replace("Model move ",""));
		
		this.generic=generic;
		this.selections=selections;
		variantCheckBoxes=new ArrayList<JCheckBox>();
		allVariants=new ArrayList<String>();
		
		ToolTipManager.sharedInstance().setInitialDelay(0);
		
		SlickerFactory sl=SlickerFactory.instance();
		JPanel container=sl.createRoundedPanel();
		container.setLayout(new GridLayout(0,1));
		container.setAlignmentX(Component.LEFT_ALIGNMENT);

		container.add(sl.createLabel("Select variants for '"+generic+"'"));
		
		//Create buttons for selecting all or none elements
		JPanel buttonsPanel=sl.createRoundedPanel();
		buttonsPanel.setLayout(new FlowLayout());
		final JButton buttonAllVariants=sl.createButton("Select all");
		final JButton buttonNoneVariants=sl.createButton("Deselect all");
		
		buttonAllVariants.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(JCheckBox b:variantCheckBoxes)
					b.setSelected(true);
				genericCheckBox.setSelected(true);
				computeSelection();
			}});
		
		buttonNoneVariants.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(JCheckBox b:variantCheckBoxes)
					b.setSelected(false);
				genericCheckBox.setSelected(false);
				computeSelection();
			}});
		buttonsPanel.add(buttonAllVariants);
		buttonsPanel.add(buttonNoneVariants);
		container.add(buttonsPanel);

		container.add(new JSeparator());
		
		
		//Create the checkbox for the generic and select it
		boolean selectedGeneric=true;
		if(selections.keySet().contains(generic))
			if(!selections.get(generic).contains(generic))
				selectedGeneric=false;
		genericCheckBox=sl.createCheckBox("Include the generic activity", selectedGeneric);
		genericCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectStuff(true);
				computeSelection();
			}});
		container.add(genericCheckBox);
				
		container.add(new JSeparator());
						
		
		//Extracts the list of all variants
		for(DatasetColumn ds:conf.getSelectedColumns()) {
			if(ds.getFullNameWithoutVariantNumber().equals(generic)) {
				allVariants.add(ds.getFullName());
				boolean isVariantSelected=true;
				if(selections.keySet().contains(generic) && 
						!selections.get(generic).contains(ds.getFullName()))
							isVariantSelected=false;
				//create the corresponding JCheckBox
				JPanel checkBoxPanel=sl.createRoundedPanel();
				checkBoxPanel.setLayout(new FlowLayout());
				final JCheckBox jcb=sl.createCheckBox(ds.getFullName(), isVariantSelected);
				jcb.setToolTipText(MultiLineTooltips.splitToolTip(arrPrefix.get(ds.getVariant()).getPrefix().toString()));
				JLabel ll=sl.createLabel("(frequence: "+conf.getColumns().get(ds).intValue()+")");
//System.out.println("col="+ds.getFullName()+" -> freq="+conf.getColumns().get(ds).intValue());
				variantCheckBoxes.add(jcb);
				jcb.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selectStuff(false);
						computeSelection();
					}});
				checkBoxPanel.add(jcb);
				checkBoxPanel.add(ll);
				container.add(checkBoxPanel);
			}
		}
		computeSelection();
		add(container);	
		
	}

	/**
	 * Determine which checkbox can be selected when a click on a button happens
	 */
	private void selectStuff(boolean flag) {
		if(flag) {
		//Generic CheckBox
		//if deselected but there are 1 or more variants selected -> keep deselected
		if(genericCheckBox.isSelected())
			if(allVariants.size()>selections.get(generic).size() && selections.get(generic).size()>0)
				//select all
				genericCheckBox.setSelected(false);
		//if deselected -> no issue
		}
		else {
		//Variants
		for(JCheckBox jcb:variantCheckBoxes) {
			//if deselected
			if(!jcb.isSelected()) {
				//if not the only one -> delect also the generic one
				 if(allVariants.size()>1)
					 genericCheckBox.setSelected(false);
				//else --> no issue
			}
			//if selected --> no issue		
		}}
	}
	
	@SuppressWarnings("deprecation")
	private void computeSelection() {
		//initialize if needed
		if(!selections.containsKey(generic))
			selections.put(generic,new ArrayList<String>());

		//update the status of the generic checkbox
		if(genericCheckBox.isSelected())
			addToHashMap(generic);
		else 
			removeFromHashMap(generic);
		
		//update the status of every checkbox for variants
		for(JCheckBox jck:variantCheckBoxes)
			if(jck.isSelected())
				addToHashMap(jck.getLabel());
			else
				removeFromHashMap(jck.getLabel());
	}

	private void addToHashMap(String el) {
		if(!selections.get(generic).contains(el)) {
			ArrayList<String> a=selections.get(generic);
			a.add(el);
			selections.replace(generic, a);
		}
	}
	
	private void removeFromHashMap(String el) {
		if(selections.get(generic).contains(el)) {
			ArrayList<String> a=selections.get(generic);
			a.remove(el);
			selections.replace(generic, a);
		}
	}
}
