package org.processmining.kpiprocessenhancementworkflow.models;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

public class PrefixContainer{

	private HashMap<String, ArrayList<PrefixFrequencyPair>> hm;
	
	public PrefixContainer() {
		
		hm=new HashMap<String,ArrayList<PrefixFrequencyPair>>();
		
	}
	
	public boolean contains(String key) {
		return hm.containsKey(key);
	}
	
	public Set<String> keySet() {
		return hm.keySet();
	}
	
	public ArrayList<PrefixFrequencyPair> getPrefixes(String key) {
		return hm.get(key);
	}
	/**
	 * Adds an activity with an empty ArrayList of pairs
	 * @param activity
	 */
	public void addActivity(String activity) {
		ArrayList<PrefixFrequencyPair> newpair=new ArrayList<PrefixFrequencyPair>();
		hm.put(activity, newpair);
	}
		
	/**
	 * Adds a new prefix to an activity
	 * @param activity
	 * @param prefix
	 */
	public void addPrefixToActivity(String activity,Set<String> prefix) {
		Set<String> toAdd=new HashSet<String>(prefix);
		ArrayList<PrefixFrequencyPair> arr=hm.get(activity);
		arr.add(new PrefixFrequencyPair(toAdd,1));
		hm.replace(activity, arr);
	}

	/**
	 * Returns the index of the specified prefix for the specified activity, or -1 if the prefix is not defined  
	 * @param activity
	 * @param prefix
	 * @return
	 */
	public int getIndexPrefix(String activity,Set<String> prefix) {
		ArrayList<PrefixFrequencyPair> arr=hm.get(activity);
		for(int i=0;i<arr.size();i++) {
			/*Set<String> toCompare=arr.get(i).getPrefix();
			 if (toCompare.size()!=prefix.size())
			    return -1;
			 HashSet<String> clone = new HashSet<String>(prefix); // just use h2 if you don't need to save the original h2
			    Iterator it = toCompare.iterator();
			    while (it.hasNext() ){
			        String = it.next();
			        if (clone.contains(A)){ // replace clone with h2 if not concerned with saving data from h2
			            clone.remove(A);
			        } else {
			            return false;
			        }
			    }*/
			if(prefix.equals(arr.get(i).getPrefix()))
			    return i; // will only return true if sets are equal
			}   
		return -1;
	}
	
	public void increaseFrequency(String activity, Set<String> prefix) {
		ArrayList<PrefixFrequencyPair> arr=hm.get(activity);
		for(PrefixFrequencyPair pfp:arr)
			if(pfp.getPrefix().equals(prefix))
				pfp.increaseFrequency();
	}
	
	public int size(String activity) {
		return hm.get(activity).size();
	}
	
	public class PrefixFrequencyPair {

		private Set<String> prefix;
		private int frequency;
		
		PrefixFrequencyPair(Set<String> prefix,int frequency) {
			this.prefix=prefix;
			this.frequency=frequency;
		}
		
		public Set<String> getPrefix() {
			return prefix;
		}
		
		void setPrefix(Set<String> prefix) {
			this.prefix=prefix;
		}
		
		public int getFrequency() {
			return frequency;
		}
		
		void setFrequency(int frequency) {
			this.frequency=frequency;
		}
		
		void increaseFrequency() {
			frequency++;
		}
}





}

