package org.processmining.kpiprocessenhancementworkflow.plugins;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.lang3.math.NumberUtils;
import org.processmining.kpiprocessenhancementworkflow.models.Rule;
import org.processmining.kpiprocessenhancementworkflow.models.RuleSet;

import weka.associations.Apriori;
import weka.core.Instance;
import weka.core.Instances;

public class AprioriWrapper extends Apriori{
	
	/**
	 * Returns the raw output of the Apriori
	 * @return
	 */
	public String getRawOutput() {
		return String.valueOf(this);
	}

	/**
	 * Parses the output of WEKA's apriori and extracts all itemsets
	 * @param outcomeToParse
	 * @return
	 */
	public ArrayList<RuleSet> extractItemSetsRules(double threshold, int totalNum, RuleSet J48Ruleset, Instances instancesClusterNonDiscretized) {
		ArrayList<RuleSet> output=new ArrayList<RuleSet>();
		
		String outcomeToParse=String.valueOf(this);
		String[] itemssets=outcomeToParse.split("Generated sets of large itemsets:")[1].split("Best rules found:")[0].split("Size of set of large itemsets L\\(");
		
		//for each itemset (starting from the largest)
		for(int i=itemssets.length-1;i>0;i--) {
			//create a ruleset
			RuleSet rs=new RuleSet();
			rs.setClassValue(1);
			
			//int numItems=Integer.parseInt(itemssets[i].substring(0,itemssets[i].indexOf(")")));
			String[] block=itemssets[i].split("Large Itemsets L\\(\\d+\\):")[1].split("\n");
			
			//This stores the indexes of those itemsets that have the highest confidence within the current itemset of lenght n
			ArrayList<Integer> bestSupportIndexes=new ArrayList<Integer>();
			int bestSupport=0;
	
			//find the rows with the top confidence
			for(int j=1;j<block.length;j++){
				String row=block[j];
		        
		        //extract the confidence 
		        int support=Integer.parseInt(row.trim().substring(row.lastIndexOf(" ")+1));
		        //skip the row if the confidence is lower than the previous value or lower than the threshold
		        if(support>=(threshold*totalNum) && support>=bestSupport) {
		        	if(support>bestSupport) {
		        		bestSupport=support;
		        		bestSupportIndexes.clear();
		        	}
		        	bestSupportIndexes.add(j);
		        }
			}
			
			for(int j:bestSupportIndexes) {
				//get the body of the row
				String bodyRaw=block[j].trim().substring(0,block[j].trim().lastIndexOf(" "));
	        
		        //System.out.println(bodyRaw+">>>"+confidence);
		        
		        //extract a set of rules (conjunction)
		        ArrayList<Rule> rules=getRulesFromItemSetBody(bodyRaw,J48Ruleset,instancesClusterNonDiscretized);
		            
		        //for each retrieved rule
		        if(rules!=null)
		        	for(Rule rule:rules) {
		        		//System.out.println(rule.getBody()+" "+rule.getType()+"  "+rule.getValue());
		        		rs.addRule(rule);
		        	}
			}
			//if no ruleSet has been created, go to the smaller itemsets
			if(rs.size()>0) {
				output.add(rs);
				break;
			}
			
		}
		
		return output;
	}
	
	
	private ArrayList<Rule> getRulesFromItemSetBody(String bodyRaw,RuleSet J48Ruleset,Instances instancesClusterNonDiscretized){
		ArrayList<String> attributeLabelforJ48Rules=new ArrayList<String>();
		for(Rule r:J48Ruleset.getList())
			attributeLabelforJ48Rules.add(r.getBody());
		
        ArrayList<Rule> rulesItem=new ArrayList<Rule>();
        while(bodyRaw.contains("=")){
        	Rule r=new Rule();
            bodyRaw+=" ";
            int indexEq=bodyRaw.indexOf("=");
            String attr=bodyRaw.substring(0,indexEq); //OK
            
            //skip if: the attribute is the class or one of the J48 decision attributes or if it generic
            if(attr.contains("CLASS") || attributeLabelforJ48Rules.contains(attr)){//!attr.contains(")")){
               if(attr.contains("CLASS"))
            	   bodyRaw="";
               else
            	   bodyRaw=bodyRaw.substring(bodyRaw.indexOf("' ")+2);
               continue;
            }
            
            boolean ends=false;
            //remove the generic version of an attibute that is in the rule list
            for(String s:attributeLabelforJ48Rules)
            	if(s.contains(attr+" (")) {
            		bodyRaw=bodyRaw.substring(bodyRaw.indexOf("' ")+2);
            		ends=true;}
            if(ends)
            	continue;
            
            bodyRaw=bodyRaw.substring(indexEq+2);
            int indexLast=bodyRaw.indexOf("' ");
            
            String value=bodyRaw.substring(0,indexLast);
            
            ArrayList<Rule> rules=checkValueAndAttr(attr,value);
            
            if(rules!=null) {
            	//i.e. if the only rule has "All" as value
            	/**if(rules.size()==0) {
            		//create two rules replacing the "All" with proper values
            		double valueForAttr=getValueForAttributeFromDataset(attr, instancesClusterNonDiscretized);
            		if(valueForAttr>=0) {
            			Rule rule0=new Rule();
            			Rule rule1=new Rule();
            			rule0.setBody(attr);
            			rule0.setType("<=");
            			rule0.setValue(valueForAttr);
            			rule1.setBody(attr);
            			rule1.setType(">=");
            			rule1.setValue(valueForAttr);
            			rulesItem.add(rule0);
            			rulesItem.add(rule1);
            			}
            	}
            	else**/
            		rulesItem.addAll(rules);
            }
            bodyRaw=bodyRaw.substring(indexLast+2);
        }
        return rulesItem;
    }
    
	/**
	 * Checks the rule in the itemset. Returns null if the rule has to be ignored. Returns an empty arraylist if the rule has value "All". Returns rules in other cases. 
	 * @param attr: the attribute in the rule
	 * @param value: the specific value in the rule
	 * @return an ArrayList of Rule(s)
	 */
    private ArrayList<Rule> checkValueAndAttr(String attr,String value){
    	ArrayList<Rule> rules=new ArrayList<Rule>();
    	Rule rule0=new Rule();
    	Rule rule1=new Rule();
        
    	String typeLower;
        String typeHigher;
        
        if(value.contains("(-inf"))
           return null;
         //TODO: se il valore è All, allora occorre significa che tutte le traces sono uguali come valore. 
        //Dunque basta andare a guardare il valore che assume la prima trace in corrispondenza di quell'attributo e sostituire
        //All con quel valore, o meglio mettere lower=higher=quel valore. 
        //Else tutto il resto che c'è sotto
        
        else if(value.equals("All")) {
            return rules;
        };       
        
        
        //Get the lower interval
        //
        String lowerStr=value.substring(1).split("-")[0];
        if(value.charAt(0)=='[')
        	typeLower=">=";
        else
            typeLower=">";
        
        if(NumberUtils.isParsable(lowerStr)) {
        rule0.setBody(attr);
        rule0.setType(typeLower);
        rule0.setValue(Double.parseDouble(lowerStr));
        rules.add(rule0);
        }
        
        //Get the higher interval
        value=value.substring(value.indexOf("-")+1,value.length());
        if(value.charAt(value.length()-1)==']')
        	typeHigher="<=";
        else
        	typeHigher="<";
        String higherStr=value.substring(0,value.length()-1);
        //System.out.println(typeHigher+" "+higher);    
       
        if(NumberUtils.isParsable(higherStr)) {
           rule1.setBody(attr);
           rule1.setType(typeHigher);
           rule1.setValue(Double.parseDouble(higherStr));
           rules.add(rule1);
        }
        
        return rules;
    }
    
    /**
     * Returns the value of the specified attribute for the first trace in the dataset. Returns -1 if the attribute is not found or in case of error.
     * @param attr
     * @param instancesClusterDiscretized
     * @return
     */
    private double getValueForAttributeFromDataset(String attr,Instances instancesClusterNonDiscretized) {
    	double output=-1;
    	for(int i=0;i<instancesClusterNonDiscretized.numAttributes();i++) {
    		System.out.println(instancesClusterNonDiscretized.attribute(i).toString());
    		if(instancesClusterNonDiscretized.attribute(i).toString().contains(attr)) {
    			//scans all possible values for the given attribute in all instances of the instaceClusterNonDiscretized, 
    			//and counts how many for each value
    			HashMap<Double,Integer> hm=new HashMap<Double,Integer>();
    			int max=0;
    			for(Instance ins:instancesClusterNonDiscretized) {
    				//considers the current value
    				double val=ins.value(i);
    				//increase the counter for such a value in the hashmap
    				if(hm.containsKey(val)) {
    					int obj=hm.get(val);
    					hm.replace(val, obj+1);
    					if(max<obj+1) {
    						max=obj+1;
    						output=val;
    					}
    				}
    				else
    					hm.put(val, 1);
    			}
    			//double value=instancesClusterNonDiscretized.firstInstance().value(i);
    			return output;
    		}
    	}
    	return -1;
    }
    
   
	
}
