package org.processmining.kpiprocessenhancementworkflow.plugins;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Scanner;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.JComponent;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeBooleanImpl;
import org.deckfour.xes.model.impl.XAttributeContinuousImpl;
import org.deckfour.xes.model.impl.XAttributeDiscreteImpl;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.framework.util.Pair;
import org.processmining.kpiprocessenhancementworkflow.confs.Configuration;
import org.processmining.kpiprocessenhancementworkflow.dialogs.AttributeSelectorDialog;
import org.processmining.kpiprocessenhancementworkflow.dialogs.J48ConfigurationPanel;
import org.processmining.kpiprocessenhancementworkflow.dialogs.MovesSelectorDialog;
import org.processmining.kpiprocessenhancementworkflow.dialogs.ShowTreeDialog;
import org.processmining.kpiprocessenhancementworkflow.models.DatasetColumn;
import org.processmining.kpiprocessenhancementworkflow.models.DatasetEntry;
import org.processmining.kpiprocessenhancementworkflow.models.LogColumn;
import org.processmining.kpiprocessenhancementworkflow.models.ModelColumn;
import org.processmining.kpiprocessenhancementworkflow.models.PrefixContainer;
import org.processmining.kpiprocessenhancementworkflow.models.PrefixContainer.PrefixFrequencyPair;
import org.processmining.kpiprocessenhancementworkflow.models.Rule;
import org.processmining.kpiprocessenhancementworkflow.models.RuleSet;
import org.processmining.models.FunctionEstimator.Type;
import org.processmining.models.guards.Expression;
import org.processmining.plugins.DataConformance.Alignment;
import org.processmining.plugins.DataConformance.Alignment.AlignmentStep;
import org.processmining.plugins.DataConformance.ResultReplay;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;
import org.processmining.plugins.balancedconformance.BalancedDataConformancePlusPlugin;
import org.processmining.prediction.J48Prediction;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;

import weka.core.Attribute;
import weka.core.DenseInstance;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffSaver;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Discretize;

public class KPEW_Plugin {
	
	File USER_DIRECTORY = org.apache.commons.io.FileUtils.getUserDirectory();
	String directory_name="/prom_KPIProcessEnhancementWorkflow";
	File BASIC_DIRECTORY=new File(String.valueOf(USER_DIRECTORY+directory_name));
	String datetime=getDateTime();
	File CURRENT_DIRECTORY = new File(String.valueOf(BASIC_DIRECTORY+"/"+datetime));
	File APRIORI_DIRECTORY = new File(String.valueOf(CURRENT_DIRECTORY+"/apriori_logs"));
	
	String FILE_PREFIXES = CURRENT_DIRECTORY+"/prefixes.txt";
	String FILE_ARFF = CURRENT_DIRECTORY+"/dataset.arff";
	//String FILE_ARFF_DISCRETIZED = CURRENT_DIRECTORY+"/dataset_discretized.arff";
	String FILE_ARFF_DISCRETIZED_FOR_CLUSTER= APRIORI_DIRECTORY+"/input";
	String FILE_APRIORI = APRIORI_DIRECTORY+"/output";
	String FILE_SELECTED_APRIORI_RULES = CURRENT_DIRECTORY+"/selected_rules.txt";
	String FILE_IMAGE_TREE = CURRENT_DIRECTORY+"/tree.png";
	
	@SuppressWarnings("deprecation")
	@Plugin(name = "KPI-driven Process Enhancement Workflow", 
			level=PluginLevel.Regular,
	        parameterLabels = { "Petri Net With Data", "Log" }, 
	        //returnLabels = { "Repaired Petrinet", "Marking" },
	        returnLabels= {"Xlog"},
	        categories=PluginCategory.Enhancement,
	        //returnTypes = { Petrinet.class, Marking.class },
	        returnTypes = { XLog.class},
	        userAccessible = true, 
	        help = "---")

	@UITopiaVariant(affiliation = "Marche Polytechnic University", 
	                author = "Emanuele Storti", 
	                email = "e.storti@univpm.it")

	public Object deriveRules(UIPluginContext context, DataPetriNet net, XLog log) throws IOException {
		
		//create directories if they do not exist
		if(!BASIC_DIRECTORY.exists())
	         BASIC_DIRECTORY.mkdir();
		CURRENT_DIRECTORY.mkdir();
		
		System.out.println("Using \""+CURRENT_DIRECTORY+"\" as current directory");
		
		//Conformance check
		BalancedDataConformancePlusPlugin alignmentPlugin = new BalancedDataConformancePlusPlugin();
		ResultReplay rr = alignmentPlugin.balancedAlignmentPlugin(context, net, log);

		Configuration conf = new Configuration();
		conf.setPrefixContainer(new PrefixContainer(),false);
		conf.setPrefixContainer(new PrefixContainer(),true);
		conf.setTable(new HashMap<String, DatasetEntry>());
		conf.setColumns(new HashMap<DatasetColumn, Integer>());
		conf.setNet(net);

		return manageInteractions(context, conf, rr, 1);
	}

	private String getDateTime() {
		LocalDateTime now = LocalDateTime.now();
		int year = now.getYear();
		int month = now.getMonthValue();
		int day = now.getDayOfMonth();
		int hour = now.getHour();
		int minute = now.getMinute();
		int second = now.getSecond();
		return year+""+month+""
				+day+"_"+hour+""+minute+""+second;
	}

	@SuppressWarnings("deprecation")
	private Object manageInteractions(UIPluginContext context, Configuration conf, ResultReplay rr, int step) {
	
		//STEP 1 - Choose indicator
		if (step == 1) {
			Object[] attributes = getAllAttributes(rr.getAlignedLog()).toArray();

			//SHOW GUI
			AttributeSelectorDialog dialog = new AttributeSelectorDialog(attributes);
			InteractionResult result = context.showWizard("Parameter selection", true, false, dialog);
			if (result == InteractionResult.NEXT) {
				conf.setIndicator(dialog.getAttributeName());
				conf.setThreshold(dialog.getThreshold());
				context.log("Analysing traces", MessageLevel.NORMAL);
				analyseData(context, rr, conf);
				return manageInteractions(context, conf, rr, 2);
			}
		}
		//STEP 2 - Select deviations
		else if (step ==2 ) {
			
			//extract generic columns > threshold
			HashMap<String, Integer> genericColumns = extractGenericColumns(conf.getTable());
			
			HashMap<String, Integer> selectedGenericColumns=new HashMap<String,Integer>();
			for(String column: genericColumns.keySet()) {
				if(genericColumns.get(column)>=conf.getThreshold())
					selectedGenericColumns.put(column, genericColumns.get(column));
			}
			conf.setSelectedGenericColumns(selectedGenericColumns);
			
			//Filter columns with frequency < threshold
			conf.setSelectedColumns(selectColumn(conf.getColumns(), conf.getThreshold()));
		
			MovesSelectorDialog dialog=new MovesSelectorDialog(context,conf);
			InteractionResult result=context.showWizard("Independent variable selection",false,false,dialog);
			if(result.equals(InteractionResult.NEXT))
			{
				//extract selected activities
				conf.setSelectedActivities(dialog.getSelectedActivities());
				return manageInteractions(context, conf, rr, 3);
			}
			else {
				//reset all
				conf.setThreshold(0);
				conf.setIndicator(null);
				conf.setPrefixContainer(new PrefixContainer(),false);
				conf.setPrefixContainer(new PrefixContainer(),true);
				conf.setTable(new HashMap<String, DatasetEntry>());
				conf.setColumns(new HashMap<DatasetColumn, Integer>());
				conf.resetAlignmentNewName();
				conf.setIndicatorValues(new HashSet<Object>());
				conf.setSelectedColumns(new HashSet<DatasetColumn>());
				conf.setSelectedGenericColumns(new HashMap<String,Integer>());
				return manageInteractions(context, conf, rr, 1);
			}
		}
		//STEP 3 - Configure Tree
		else if (step == 3) {
			context.log("Creating instances...", MessageLevel.NORMAL);
			try {
				savePrefixes(new File(FILE_PREFIXES), conf);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
			J48ConfigurationPanel j48configuration=new J48ConfigurationPanel(rr.getAlignedLog().size(),conf.getSelectedActivities().size());
			InteractionResult resultJ48Panel=context.showWizard("Decision tree configuration panel",false,false,j48configuration);
			
			if(!resultJ48Panel.equals(InteractionResult.NEXT))
			{
				//reset configuration
				conf.setSelectedActivities(new ArrayList<String>());
				return manageInteractions(context, conf, rr, 2);
			}
			else {
				conf.putDecisionTreeConfigurationItem("binarySplit", j48configuration.binaryTree());
				conf.putDecisionTreeConfigurationItem("unpruned", !j48configuration.prunedTree());
				conf.putDecisionTreeConfigurationItem("confidenceFactor", j48configuration.getConfidenceThreshold());
				conf.putDecisionTreeConfigurationItem("minNumObj", j48configuration.getMinNumInstancePerLeaf());
				conf.putDecisionTreeConfigurationItem("numFolds", j48configuration.getNumFoldErrorPruning());
				conf.putDecisionTreeConfigurationItem("apriori", j48configuration.apriori());
				conf.putDecisionTreeConfigurationItem("apriori_Threshold", j48configuration.aprioriThreshold());
				conf.putDecisionTreeConfigurationItem("apriori_DeltaThreshold", j48configuration.aprioriDeltaThreshold());
				
				//lancio il thread
				
				//creo dialog con OK disabilitato (verrà abilitato quando ha finito)
				//se ritorna OK, allora vado avanti
				//se ritorna NO, blocco il thread
				
				return manageInteractions(context, conf, rr, 4);	
			}
			}
		//STEP 4 - Generate Tree
		else if(step==4) {
			context.log("[decision tree] Generating the input dataset...", MessageLevel.NORMAL);
			J48Prediction p = buildInstance(conf, rr);
			/*System.out.println("confidence="+p.getConfidenceThreshold());
			System.out.println("minnuminstanceperleaf="+ p.getMinNumInstancePerLeaf());
			System.out.println("numfolderror="+p.getNumFoldErrorPruning());*/
			p.setBinarySplit((boolean)conf.getDecisionTreeConfigurationItem().get("binarySplit"));			
			p.setUnpruned((boolean)conf.getDecisionTreeConfigurationItem().get("unpruned"));	
			p.setConfidenceFactor((float)conf.getDecisionTreeConfigurationItem().get("confidenceFactor"));
			p.setMinNumObj((int)conf.getDecisionTreeConfigurationItem().get("minNumObj"));
			p.setNumFolds((int)conf.getDecisionTreeConfigurationItem().get("numFolds"));
			
			
			try {
				p.setCrossValidate(true);
				//create the tree
				p.createAndSetTree(new Object[] {});
				//serialize the table
				p.saveInstances(new File(FILE_ARFF));

				float confidence = p.getConfidenceThreshold();
				double fmeasure = p.computeFMeasure();
				double quality = p.computeQualityMeasure();
				System.out.println("confidence=" + confidence);
				System.out.println("fmeasure=" + fmeasure);
				System.out.println("quality=" + quality);

			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			//Generate the tree
			context.log("[decision tree] showing the tree...", MessageLevel.NORMAL);
			JComponent tree = p.getVisualization();
			tree.setSize(1200,1200);
			saveTreeFile(tree);
			ShowTreeDialog dialogTree = new ShowTreeDialog(context, tree);
			
			InteractionResult resultTree = context.showWizard("Tree visualisation", false, true, dialogTree);
			
			
			if (resultTree == InteractionResult.PREV) {
				//reset configuration
				conf.deleteDecisionTreeConfiguration();
				return manageInteractions(context, conf, rr, 3);
			
			} else if (resultTree == InteractionResult.FINISHED) {
		
				//for each branch in the tree, define a RuleSet
				for (Pair<String, Expression> pair : p.getExpressionsAtLeaves()) {
					String classValue = pair.getFirst();
					String[] expressions = pair.getSecond().toString().split("&&");
					//pair.getSecond().evaluate(h)
					RuleSet ruleset=new RuleSet();
					ruleset.setClassValue(Integer.parseInt(classValue));
					for (String s : expressions) {
						Rule rule=Rule.getRuleFromExpression(s.trim().replace("(", "").replace(")", "").replace(" + ", " ").replace("$28", "(")
								.replace("$29", ")"));
						ruleset.addRule(rule);
					}
					conf.addRuleSet(ruleset);
				
				}
				//Execute the APRIORI algorithm if requested
				if((boolean)conf.getDecisionTreeConfigurationItem().get("apriori")) {
					APRIORI_DIRECTORY.mkdir();

					double threshold=(double)conf.getDecisionTreeConfigurationItem().get("apriori_Threshold");
					double deltaThreshold=(double)conf.getDecisionTreeConfigurationItem().get("apriori_DeltaThreshold");
					apriori(conf,rr,threshold,deltaThreshold);
					
				}
			
				return manageInteractions(context, conf, rr, 5);
			} //end interactionresult finished
		}
		// STEP 5 - last steps
		else if (step == 5) {
			XLog repairedLog = XFactoryRegistry.instance().currentDefault().createLog();
			
			//for each trace in the original log
			for(XTrace trace: rr.getAlignedLog()) {
				Object value=getIndicatorValue(trace, conf);
				String traceid=XConceptExtension.instance().extractName(trace);
				//extract its alignment
				Alignment alignment=rr.getAlignmentByTraceName(traceid);
				
				RuleSet myRuleSet=null;
				//find the cluster for the trace at hand
				for(RuleSet rule:conf.getRulesets())
					if (checkExpression(traceid, rule, conf)) {
						myRuleSet=rule;
					}
				
				
				int classValue=myRuleSet.getClassValue();
				
				boolean repairAll=false;
				//completely repair traces classified with a bad KPI or misclassified as good 
				if(classValue==0 || (long)value!=classValue) 
					repairAll=true;
				
				//completely repair traces that are classified with a good KPI but do not satify the additional set of rules
				else {
					ArrayList<RuleSet> linkedAprioriRulesets=null;
					//if there are apriori rules
					if(conf.getMappedAprioriRulesets(myRuleSet)!=null) {
						linkedAprioriRulesets=conf.getMappedAprioriRulesets(myRuleSet);
						//at least one of them must be satisfied
						boolean checkedAprioriRulesSatisfaction=false;
						for(RuleSet r:linkedAprioriRulesets) {
							if(checkExpression(traceid, r, conf))
								checkedAprioriRulesSatisfaction=true;
								break;
							}
						if(!checkedAprioriRulesSatisfaction)
							repairAll=true;
						}
					}
				
				//repair the trace if classified with a good KPI and if it satisfies the new set of rules
				XTrace repairedTrace=repairTrace(trace,traceid,alignment,myRuleSet,conf, repairAll);					
				
				
				//add the repaired trace to repairedLog
				if (repairedTrace.size()>0)
					//repairedLog.add(addMissingTimeStamp(repairedTrace));
					repairedLog.add(repairedTrace);
		}
			
			return repairedLog;
			//END - REPAIR MODEL
			//Uma_RepairModel_Plugin modelRepairPlugin=new Uma_RepairModel_Plugin();
			//return modelRepairPlugin.repairModel(context, repairedLog, conf.getNet());
				}
		return null;
	}

private void saveTreeFile(JComponent tree) {
	    try {
	    	BufferedImage i = new BufferedImage(1200, 1200,BufferedImage.TYPE_INT_RGB);
	    	
	    	Graphics2D g2=i.createGraphics();
	    	tree.paint(g2);
	    	//Graphics g = i.getGraphics();
	        //g.setColor(Color.BLACK);
	        //paint(g);
	        ImageIO.write(i, "jpeg", new File(FILE_IMAGE_TREE));
	        } 
	    catch (IOException e) {
	        e.printStackTrace();
	    }
	}
		
	

private void apriori(Configuration conf, ResultReplay rr, Double threshold, Double deltaThreshold) {
	//APRIORI******************
	//this should be optimized
	// TODO allargare la ruleset oppure sostituirla con una serie di altre rulesets? Alla fine
	// TODO la parte relativa al apriori vale solo per quelli di cluster=1 che sono stati correttamente classificati
	//la questione è legata al fatto che trovare itemsets significa anche allargare l'albero (trade accuracy with recognition of other correlated attributes) 
	 int apriori_num_executions=0;
	
	 Instances instances=extractAllInstances(conf, rr);
	
	for(RuleSet ruleset:conf.getRulesets()) {
		//for rules leading to a HIGH kpi
		if(ruleset.getClassValue()==1) {
			//initialize the cluster
			ArrayList<Attribute> listAttribute=createAttributeList(getTypes(conf), new HashMap<String, Set<String>>() , conf.getIndicatorValues().toArray());
			/*ArrayList<Attribute> listAttribute2=new ArrayList<Attribute>();
			for(Attribute attrname:listAttribute)
				if(!attrname.toString().contains("CLASS"))
					listAttribute2.add(attrname);
			*/
			Instances instancesCluster=new Instances("relation",listAttribute,100);
			//instancesCluster.setClassIndex(listAttribute.size()-1);
			
			//for each trace retrieve which rule is applied
			for(XTrace trace:rr.getAlignedLog()) {
				String traceid=XConceptExtension.instance().extractName(trace);
				if(checkExpression(traceid, ruleset, conf)) {
					// (ho mappato le instances alle traces nella conf)
					// qui creo un cluster andando a considerare le traces correctly classified
					Instance inst=new DenseInstance(listAttribute.size());
					int traceClass=Integer.parseInt(getIndicatorValue(trace, conf).toString());
					for(Attribute attr:listAttribute) {
						int num=0;
						//for class attribute
						if(attr.toString().contains("CLASS")) 
							//continue;
							num=traceClass+1;
						//for attributes with variants
						else if(attr.toString().contains("("))
							num=getNumMovesinTracesForVariableWithPrefixes(traceid, attr.name(), conf);
						//for generic attributes
						else	
							num=getNumMovesinTraces(traceid, attr.name(), conf);
						inst.setValue(attr, num);
						//add the instance to the general list in the Configuration class
					}
					//add the trace to the dataset only if the class value is "1"
					//inst.setValue(instancesCluster.classAttribute(), Double.parseDouble(getIndicatorValue(trace, conf).toString()));
					if(traceClass==1)
						instancesCluster.add(inst);
				}	
			}		
				try {
					//get the number of elements in the cluster
					int sizeInstanceCluster=instancesCluster.size();
					//adds all other instances for discretization purposes (so to consider all possible values)
					instancesCluster.addAll(instances);
					instancesCluster.setClassIndex(listAttribute.size()-1);
					
					
					// discretize the dataset for the instanceCluster
					Discretize filt=new Discretize();
					filt.setInputFormat(instancesCluster);
					Instances allInstancesDiscretized=Filter.useFilter(instancesCluster, filt);
					
					//saveInstances(new File(FILE_ARFF_DISCRETIZED),allInstancesDiscretized);
					
					ArrayList<Attribute> arrAtt=new ArrayList<Attribute>();
					for(int i=0;i<allInstancesDiscretized.numAttributes();i++)
						arrAtt.add(allInstancesDiscretized.attribute(i));
					
					//extract again the instances for the specific cluster at hand
					Instances instancesForApriori=new Instances("relation",arrAtt,100);
					for(int i=0;i<sizeInstanceCluster;i++)
						instancesForApriori.add(allInstancesDiscretized.get(i));
					saveInstances(new File(FILE_ARFF_DISCRETIZED_FOR_CLUSTER+apriori_num_executions+".arff"),instancesForApriori);
					
					// generate frequent itemset through Apriori (WEKA)
					AprioriWrapper ap=new AprioriWrapper();
					ap.setOutputItemSets(true);
					ap.setDelta(deltaThreshold);
					//ap.setClassIndex(instancesClusterDiscretized.classIndex());
					ap.buildAssociations(instancesForApriori);
					
					//save the output of the apriori to a file
					saveAprioriOutput(new File(FILE_APRIORI+apriori_num_executions+".txt"), ap.getRawOutput(),ruleset);
					
					// set a threshold
					int numtotal=instancesForApriori.size();
					ap.setLowerBoundMinSupport(threshold);
					
					//parse and analyse the itemset, and extract rules
					ArrayList<RuleSet> newRules=ap.extractItemSetsRules(threshold,numtotal,ruleset,instancesCluster);
					
					//link these ruleset to the J48's ruleset
					conf.mapTreeRuleSetWithAprioriRuleset(ruleset,newRules);
					save_AprioriOutputFile(ruleset,newRules);
				}
				catch(Exception e) { 
					e.printStackTrace();
				}
				apriori_num_executions++;
				} // fine if (class==1)
		
	} //for each ruleset of the tree
}
	
	
		protected ArrayList<Attribute> createAttributeList(Map<String, Type> attributeType,
			Map<String, Set<String>> literalValues, Object[] outputClasses) {
		ArrayList<Attribute> attributeList = new ArrayList<>(attributeType.keySet().size() + 1);
		/*
		 * For each entry <String, Type> in map, depending on the Type of the attribute,
		 * add a new Attribute (attribute name, format for values of said attribute) to
		 * ArrayList<Attribute> attributeList
		 */
		for (Entry<String, Type> entry : attributeType.entrySet()) {
			switch (entry.getValue()) {
				case LITERAL :
					attributeList
							.add(new Attribute(entry.getKey(), new ArrayList<>(literalValues.get(entry.getKey()))));
					break;
				case TIMESTAMP :
					//TODO FM, why do we need this timeformat here?
					attributeList.add(new Attribute(entry.getKey(), "yyyy-MM-dd'T'HH:mm:ss"));
					break;
				case DISCRETE : // Do the same as case CONTINOUS
				case CONTINUOS :
					attributeList.add(new Attribute(entry.getKey()));
					break;
				case BOOLEAN :
					attributeList.add(new Attribute(entry.getKey(), Arrays.asList("T", "F")));
			}
		} // END for(Entry<String, Type> entry : map.entrySet())

		// Initialize the classValues ArrayList to an ArrayList<String> of size outputValuesAsObjects.length
		ArrayList<String> classValues = new ArrayList<String>(outputClasses.length);
		// Add the predefined nullValue to classValues
		classValues.add("NOT SET");
		BiMap<Object, Integer> classIndexMap = HashBiMap.create();
		classIndexMap.put("NOT SET", 0);
		/*
		 * Populate the classValues array with unique identifiers for class values and
		 * populate 'mapping' with mappings from the unique String identifier of a class
		 * value to the actual class value Object
		 */

		BiMap<String, Object> classMapping = HashBiMap.create();
		for (int i = 0; i < outputClasses.length; i++) {
			/*
			 * For each Object (Transition) in outputValues, convert the Transition to a
			 * string 'value', replacing newlines with spaces and put it in classValues. If
			 * ArrayList<String> classValues already contains that value, concatenate an
			 * integer to it until it's not in classValues. Finally, add the mapping (String
			 * value, Object outputValues[i]) to Map<String,Object> 'mapping'
			 */
			String value = outputClasses[i].toString().replace('\n', ' ');
			int n = 0;
			while (classValues.contains(value)) {
				n++;
				value = value + n;
			}
			classValues.add(value);
			classIndexMap.put(outputClasses[i], i + 1);
			classMapping.put(value, outputClasses[i]);
		}

		// Define a new attribute ("CLASS",classValues) containing the unique textual representations of target Transitions
		Attribute classAttribute = new Attribute("CLASS", classValues);

		/*
		 * Add the ("CLASS",classValues) attribute to to the end of attributeList. -->
		 * We now have an attribute to uniquely identify the target transitions <-- Use
		 * as class for classification!
		 */
		attributeList.add(classAttribute);

		return attributeList;
	}

	private XTrace repairTrace(XTrace oldTrace,String traceid, Alignment alignment, RuleSet ruleset,
			Configuration conf, boolean repairAll) {
		XFactory factory =  XFactoryRegistry.instance().currentDefault();
		XTrace repairedTrace=factory.createTrace();
		XConceptExtension.instance().assignName(repairedTrace, traceid);
		
		Iterator<AlignmentStep> alignIter = alignment.alignmentStepIterator();
		Iterator<XEvent> existingTraceIter = oldTrace.iterator();
		String trace_id = XConceptExtension.instance().extractName(oldTrace);
		XEvent eventToAdd = null;

		// for each step of the alignment
		int stepNum = 0;  // the step of the trace
		int moveNum = 0;  //counts the number of moves in the trace
		ArrayList<RuleSet> linkedAprioriRulesets=conf.getMappedAprioriRulesets(ruleset);
		if(linkedAprioriRulesets==null)
			linkedAprioriRulesets=new  ArrayList<RuleSet>();
		
		while (alignIter.hasNext()) {
			AlignmentStep step = alignIter.next();
			
			switch (step.getType()) {
				case L :
					eventToAdd = createEvent(null, step);
					existingTraceIter.next();
					try {
						//REPAIR: if -> the class is 0 || (there is no rule with >=0 for the activityNewName of this step)  
						if (repairAll || (!isStepInThisRuleset(moveNum, trace_id, ruleset, conf) && !isStepInThisSetOfRulesets(moveNum, trace_id, linkedAprioriRulesets, conf)))
							eventToAdd = null;
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					moveNum++;
					break;
				case LMGOOD :
					eventToAdd = createEvent(existingTraceIter.next(), step);
					break;
				case LMNOGOOD :
					XEvent nextEvent = existingTraceIter.next();
					eventToAdd = createEvent(nextEvent, step);
					if (true) {
						eventToAdd = createEvent(nextEvent, step);
					}
					break;
				case MREAL :
					//REPAIR: if -> the class is 0 || (there is no rule with >=0 for the activityNewName of this step)
					if (repairAll || (!isStepInThisRuleset(moveNum, trace_id, ruleset, conf) && !isStepInThisSetOfRulesets(moveNum, trace_id, linkedAprioriRulesets, conf)))
						eventToAdd = createEvent(null, step);
					else
						//do NOT REPAIR
						eventToAdd = null;
					moveNum++;
					break;
				case MINVI :
					eventToAdd = null;
					break;
				default :
					assert (false);
					break;
			}
			if (eventToAdd != null)
				repairedTrace.add(eventToAdd);
			//increase the step counter
			stepNum++;
		}
		return repairedTrace;
	}

	/**
	 * Check if there is a rule (of type ">=") on the same attribute name (plus
	 * prefix) of the current step. In case of multiple rules like this one, check their conjunction
	 * 
	 * @param step
	 * @param ruleset
	 * @param conf
	 * @return
	 */
	private boolean isStepInThisRuleset(int stepNum, String trace_id, RuleSet ruleset, Configuration conf) {
		for (Rule rule : ruleset.getList()) {
			if(conf.getNewName(trace_id, stepNum).contains(rule.getBody())) {
				//TODO: questa condizione va migliorata. Infatti bisogna valutare la condizione veramente, non vedere se il type è > o >=
				if ((rule.getType().equals(">") || rule.getType().equals(">=")))
				//	check=true;
					return true;
				//else return false;
			}
		}
		return false;
	}

	/**
	 * Check if there exists a rule (within one of the input ruleset) that is satisfied.
	 * @param stepNum
	 * @param trace_id
	 * @param ruleset
	 * @param conf
	 * @return
	 */
	private boolean isStepInThisSetOfRulesets(int stepNum, String trace_id, ArrayList<RuleSet> ruleset, Configuration conf) {
		for(RuleSet rs:ruleset) {
			/*for(Rule r:rs.getList()) {
				RuleSet rs_OR=new RuleSet();
				rs_OR.addRule(r);
				if(isStepInThisRuleset(stepNum, trace_id, rs_OR, conf))
					return true;
			}*/
			if(isStepInThisRuleset(stepNum,trace_id,rs,conf))
				return true;
		}
		return false;
	}
	
	private void analyseData(UIPluginContext context, ResultReplay rr, Configuration conf) {
		XLog log = rr.getAlignedLog();

		int counterTrace = 0;
		
		// read the ResultReplay trace by trace
		for (XTrace t : log) {
			DatasetEntry entry = new DatasetEntry();
			Set<String> currentPrefix = new HashSet<String>();
			String trace = XConceptExtension.instance().extractName(t);
			
			
			Alignment al = rr.getAlignmentByTraceName(XConceptExtension.instance().extractName(t));
			
			//extract the value for the chosen indicator
			XEvent lastEvent = t.get(t.size() - 1);
			XAttribute att = lastEvent.getAttributes().get(conf.getIndicator());

			if (att instanceof XAttributeContinuous)
				conf.addIndicatorValue(((XAttributeContinuous) att).getValue());
			else if (att instanceof XAttributeDiscrete)
				conf.addIndicatorValue(((XAttributeDiscrete) att).getValue());
			else if (att instanceof XAttributeLiteral)
				conf.addIndicatorValue(((XAttributeLiteral) att).getValue());

			//for each alignment step
			//int counterEvent=0;
			for (AlignmentStep als : al) {
				//extract the execution steps
				ExecutionStep activity;
				//ExecutionStep modelActivity = als.getProcessView();
				DataAwareStepTypes type = als.getType();
				if (type.equals(DataAwareStepTypes.MREAL))
					activity = als.getProcessView();
				else
					activity=als.getLogView();
						
				int variant = 0;
				
				//if type is sync or log move or model move
				if ((type.equals(DataAwareStepTypes.LMGOOD) || type.equals(DataAwareStepTypes.L)
						|| type.equals(DataAwareStepTypes.MREAL)) && activity.getActivity() != null) {
					//int variant;

					
			/**		if(type.equals(DataAwareStepTypes.L))
						if(activity.toString().contains("8") && !(activity.toString().contains("18")))
						{	eightsLog=eightsLog+1;
							eights=eights+1;
							System.out.println("L->"+activity.toString());
					}
					if(type.equals(DataAwareStepTypes.MREAL))
						if(activity.toString().contains("8") && !(activity.toString().contains("18")))
						{	eightsModel=eightsModel+1;
							eights=eights+1;
							System.out.println("M->"+activity.toString());
					}
				**/	
					
					
			if( type.equals(DataAwareStepTypes.L) || type.equals(DataAwareStepTypes.MREAL)){
				
					//if the prefix container does not contain the activity
					boolean isModelMove=false;
					if(type.equals(DataAwareStepTypes.MREAL)) 
						isModelMove=true;
					
						
					//TODO escludo la model move dal calcolo dei prefix
					if (!conf.getPc(isModelMove).contains(activity.getActivity())) {
						//add the activity 
						conf.addActivityToPc(activity.getActivity(),isModelMove);
						//add the prefix (with default frequency 1)
						conf.addPrefixToActivityPc(activity.getActivity(), currentPrefix,isModelMove);
					}
					// else (i.e. the activity is in the container OR the activity is not a model move) check if the prefix is already defined
					else {
						//if the prefix already exists
						if (conf.getPc(isModelMove).getIndexPrefix(activity.getActivity(), currentPrefix) >= 0) {
							// increase the frequency of the prefix for the activity by 1
							conf.increaseFrequencyPc(activity.getActivity(), currentPrefix,isModelMove);
							//extract the variant number
							variant = conf.getPc(isModelMove).getIndexPrefix(activity.getActivity(), currentPrefix);
						}
						//otherwise, register the prefix as a variant and assign the variant number accordingly
						else {
							conf.addPrefixToActivityPc(activity.getActivity(), currentPrefix,isModelMove);
							variant = conf.getPc(isModelMove).size(activity.getActivity()) - 1; //EDIT tolto il meno 1
						}
					}
					} //end if !modelmove

				
					
					// update the prefix with the current activity, if type is sync or log. If it is a model move, there is no need.
					if (!type.equals(DataAwareStepTypes.MREAL)) {
						currentPrefix = updatePrefix(currentPrefix, activity.getActivity());
					}

					//update the table: create a new column for a log or a model move
					DatasetColumn column = null;
					if (type.equals(DataAwareStepTypes.MREAL)) {
						column = new ModelColumn(activity.getActivity(), variant);
					} else if (type.equals(DataAwareStepTypes.L)) {
						column = new LogColumn(activity.getActivity(), variant);
					} else
						continue;

					//in case of a log or model move 
					//add the column to the Set of columns if it is new or otherwise update its frequency
					boolean check = false;
					// set check=true if the columns is already in the columns
					for (DatasetColumn dd : conf.getColumns().keySet()) {
						//check also if the type is the same (log or model)
						if (dd.getClass().equals(column.getClass()) && dd.equals(column)) {
							conf.replaceColumn(dd, conf.getColumns().get(dd) + 1);
							check = true;
							break;
						}
					}
					if (!check)
						conf.putColumns(column, 1);

					//and update the entry in the dataset
					try {
						DatasetColumn identical = entry.containsColumn(column); //EDIT: chiarificato questo punto e passato il datasetcolumn di entry e non column direttamente
						if (identical != null) {
							int value = entry.getValue(identical); //EDIT prima era int value = entry.getValue(column) e dentro controllo anche il variant; 
							if (value > 0)
								entry.setValue(identical, value + 1);
						} else {
							entry.addColumn(column);
						}
					} catch (Exception e) {
						e.printStackTrace();
					}

					//map the alignment step with the new name -> useful in the repair log step
					String newName = "";
					if (type.equals(DataAwareStepTypes.MREAL))
						newName = "Model move " + activity.getActivity() + " (" + variant + ")";
					else if (type.equals(DataAwareStepTypes.L))
						newName = "Log move " + activity.getActivity() + " (" + variant + ")";
					conf.putAlignmentNewName(trace, newName);
				} //end check type - if

				//decide if to consider the Log or the Sync
				//counterEvent++;
			} //end step - for	

			//add a new table entry for the trace at hand
			conf.putTable(trace, entry);
			counterTrace++;
		} //end trace		

	}

	private HashMap<String, Integer> extractGenericColumns(HashMap<String, DatasetEntry> table) {
		HashMap<String, Integer> genericColumns = new HashMap<String, Integer>();
		
		//for each trace
		for (String id : table.keySet()) {
			//for each of its columns
			for (DatasetColumn c : table.get(id).getColumns()) {
				//get the generic column name and its frequency
				String columnName = c.getFullNameWithoutVariantNumber();
		
				
				int frequency = table.get(id).getValue(c);
				
				//if already set in genericColumns, just increase the frequency
				if (genericColumns.containsKey(columnName)) {
					int oldValue = genericColumns.get(columnName);
					genericColumns.replace(columnName, oldValue + frequency);
				} else
					genericColumns.put(columnName, frequency);
			}

		}
		return genericColumns;

	}

	private J48Prediction buildInstance(Configuration conf, ResultReplay rr) {
		J48Prediction p;
		XLog originalLog = rr.getAlignedLog();

		//set the type DISCRETE for all columns (both selected and generic)
		HashMap<String, Type> hmTypes = getTypes(conf);

		HashMap<String, Integer> smallerGenericColumns = new HashMap<String, Integer>();
		for (String s : conf.getSelectedGenericColumns().keySet()) {
			//if(s.contains("Log move Adjustment Charge") || s.contains("Log move Block Purchase Order Item") || s.contains("Log move Create Purchase Order Item") || s.contains("Log move Create Purchase Requisition Item") || s.contains("Log move Delete Purchase Requisition Item") || s.contains("Log move Send Overdue Notice") || s.contains("Log move Send Purchase Order Update") || s.contains("Log move Vendor Creates Invoice") || s.contains("Model move Create Purchase Requisition Item"))
			if (conf.getSelectedActivities().contains(s))
				smallerGenericColumns.put(s, conf.getSelectedGenericColumns().get(s));
		}

		//add the selected column
		Set<DatasetColumn> smallerSelection = new HashSet<DatasetColumn>();
		for (DatasetColumn dc : conf.getSelectedColumns()) {
			if (conf.getSelectedActivities().contains(dc.getFullName()))
				//if(dc.getActivity().contains("Adjustment Charge") || dc.getActivity().contains("Block Purchase Order Item") || dc.getActivity().contains("Create Purchase Order Item") || dc.getActivity().contains("Create Purchase Requisition Item") || dc.getActivity().contains("Delete Purchase Requisition Item") || dc.getActivity().contains("Send Overdue Notice") || dc.getActivity().contains("Send Purchase Order Update") || dc.getActivity().contains("Vendor Creates Invoice") || dc.getActivity().contains("Create Purchase Requisition Item"))
				smallerSelection.add(dc);
		}

		Object[] objValues = conf.getIndicatorValues().toArray();
		//initialize J48Prediction (types mapping nomi attributi e tipi, literalvalues per mappare attributi letterali con le possibili labels ,possibili valori della KPI,  metto vuoto per timeinterval)

		p = new J48Prediction(hmTypes, new HashMap<String, Set<String>>(), objValues, "predictor", 100,
				new HashSet<String>());

		//ADD INSTANCES

		//For each trace, build the instance to be added to the predictor
		for (XTrace trace : originalLog) {
			HashMap<String, Object> instance = new HashMap<String, Object>();
			//get the concept name
			String trace_id = XConceptExtension.instance().extractName(trace);
			//get the desired indicator value from the last event of the trace
			Object ind_value = getIndicatorValue(trace, conf);

			DatasetEntry entry = conf.getTable().get(trace_id);

			//for each column 
			for (DatasetColumn dc : smallerSelection) {
				if (entry.containsColumn(dc) != null)
					instance.put(dc.getFullName(), entry.getValue(dc));
				else
					instance.put(dc.getFullName(), 0);
			}
			//detect the number of moves for the i-th generic column for the specific trace
			for (String s : smallerGenericColumns.keySet()) {
				int numMovesinTraces = getNumMovesinTraces(trace_id, s, conf);
				instance.put(s, numMovesinTraces);
			}

			//add the instance for the i-th trace to the dataset
			try {
				p.addInstance(instance, ind_value, 1);
			} catch (Exception e) {
				e.printStackTrace();
			}

		} // end for trace
		return p;
	}

	/**
	 * Extract the indicator value from a trace
	 * 
	 * @param trace
	 * @param conf
	 * @return
	 */
	private Object getIndicatorValue(XTrace trace, Configuration conf) {
		Object ind_value = new Object();
		XAttribute att = trace.get(trace.size() - 1).getAttributes().get(conf.getIndicator());
		if (att instanceof XAttributeContinuous)
			ind_value = ((XAttributeContinuous) att).getValue();
		else if (att instanceof XAttributeDiscrete)
			ind_value = ((XAttributeDiscrete) att).getValue();
		else if (att instanceof XAttributeLiteral)
			ind_value = ((XAttributeLiteral) att).getValue();

		return ind_value;
	}

	private HashMap<String, Type> getTypes(Configuration conf) {
		HashMap<String, Type> hmTypes = new HashMap<String, Type>();
		for (String name : conf.getSelectedActivities())
			hmTypes.put(name, Type.DISCRETE);
		return hmTypes;
	}

	/**
	 * Extracts the number of moves in a trace with name "trace_id" for a certain
	 * attribute "s"
	 * 
	 * @param trace_id
	 * @param s
	 * @param conf
	 * @return
	 */
	private int getNumMovesinTraces(String trace_id, String s, Configuration conf) {
		int numMovesinTraces = 0;
		//see if there is or not and with which frequency (sum up all the variants' frequencies)
		for (DatasetColumn c : conf.getTable().get(trace_id).getColumns())
			if (c.getFullNameWithoutVariantNumber().equals(s))
				numMovesinTraces += conf.getTable().get(trace_id).getValue(c);
		return numMovesinTraces;
	}

	/**
	 * Extracts the number of moves in a trace with name "trace_id" for a certain
	 * attribute "s"
	 * 
	 * @param trace_id
	 * @param s
	 * @param conf
	 * @return
	 */
	private int getNumMovesinTracesForVariableWithPrefixes(String trace_id, String s, Configuration conf) {
		for (DatasetColumn c : conf.getTable().get(trace_id).getColumns())
			if (c.getFullName().equals(s))
				return conf.getTable().get(trace_id).getValue(c);
		return 0;
	}

	
	/**
	 * Adds the new activity in the prefix set if it is not already added
	 * 
	 * @param i
	 * @param prefix
	 * @param newActivity
	 * @return
	 */
	private static Set<String> updatePrefix(Set<String> prefix, String newActivity) {
		Set<String> output = prefix;
		if (!prefix.contains(newActivity))
			output.add(newActivity);

		return output;
	}

	private Set<DatasetColumn> selectColumn(HashMap<DatasetColumn, Integer> columns, int threshold) {
		HashSet<DatasetColumn> selectedColumns = new HashSet<DatasetColumn>();
		//select only those with frequency > threshold
		for (DatasetColumn d : columns.keySet())
			if (columns.get(d) >= threshold)
				selectedColumns.add(d);

		return selectedColumns;
	}

	private Set<String> getAllAttributes(XLog log) {
		Set<String> output = new HashSet<String>();

		for (XTrace t : log) {
			XEvent lastEvent = t.get(t.size() - 1);
			XAttributeMap map = lastEvent.getAttributes();
			for (String name : map.keySet()) {
				XAttribute a = map.get(name);
				try {
					double a1 = Double.parseDouble(a.toString());
					output.add(name);
				} catch (Exception e) {
				}
			}
		}

		return output;
	}

	/**
	 * Check if a rule, composed by a set of expressions, is satisfied or not by a
	 * trace
	 * 
	 * @param traceid
	 * @param rule
	 * @param conf
	 * @return
	 */
	private boolean checkExpression(String traceid, RuleSet ruleset, Configuration conf) {
		//for each specific subrule of a rule
		for (Rule singleRule : ruleset.getList()) {
			String variable = singleRule.getBody();
			String type = singleRule.getType();
			double threshold = singleRule.getValue();
			if (!checkSingleExpression(traceid, variable, type, (double) threshold, conf))
				return false;
		}
		return true;
	}

	/**
	 * Check if a single expression is satisfied by the trace at hand
	 * 
	 * @param traceid
	 * @param variable
	 * @param type
	 * @param threshold
	 * @param conf
	 * @return
	 */
	private boolean checkSingleExpression(String traceid, String variable, String type, Double threshold,
			Configuration conf) {
		boolean output = false;

		DatasetEntry entry = conf.getTable().get(traceid);
		int numMoves;
		//find the number of moves for the attribute at hand and the trace at hand
		if (variable.contains("(")) { // in case of a "log" or "move" DatasetColumn
			String str = variable.split("\\(")[1].replace(")", "").trim();
			int variant = Integer.parseInt(str);
			String name = variable.split("\\(")[0].replace("Log move", "").replace("Model move", "").trim();
			DatasetColumn col;
			if (variable.contains("Log move")) // Log move
				col = new LogColumn(name, variant);
			else { //Movel move
				col = new ModelColumn(name, variant);
			}
			//if the entry contains the columns, then extract the number of moves
			if (entry.containsColumn(col) != null)
				numMoves = entry.getValue(col);
			//else set it to 0
			else
				numMoves = 0;
		} else { // in case of a "generic column"
			numMoves = getNumMovesinTraces(traceid, variable, conf);
		}

		//check if the condition is satisfied
		switch (type) {
			case (">") :
				if (numMoves > threshold)
					output = true;
				break;
			case (">=") :
				if (numMoves >= threshold)
					output = true;
				break;
			case ("<") :
				if (numMoves < threshold)
					output = true;
				break;
			case ("<=") :
				if (numMoves <= threshold)
					output = true;
				break;
			default :
		}
		return output;
	}

	private static XEvent createEvent(XEvent event, AlignmentStep step) {
		String activityName = step.getProcessView().getActivity();
		if (activityName == null)
			activityName = step.getLogView().getActivity();
		XEvent newEvent = XFactoryRegistry.instance().currentDefault().createEvent();
		if (event == null) {
			XLifecycleExtension.StandardModel activityEventType = StandardModel.COMPLETE;
			//XLifecycleExtension.instance().assignStandardTransition(newEvent, activityEventType);
		} else {

			if (XLifecycleExtension.instance().extractStandardTransition(event) != null)
				XLifecycleExtension.instance().assignStandardTransition(newEvent, StandardModel.COMPLETE);
			//if(XTimeExtension.instance().extractTimestamp(event)!=null)
			//XTimeExtension.instance().assignTimestamp(newEvent, XTimeExtension.instance().extractTimestamp(event));
		}
		XConceptExtension.instance().assignName(newEvent, activityName);

		for (Entry<String, Object> entry : step.getProcessView().entrySet()) {
			String key = entry.getKey();
			Object value = entry.getValue();
			XAttribute attrib;
			if (value instanceof Boolean)
				attrib = new XAttributeBooleanImpl(key, (Boolean) value);
			else if (value instanceof Integer)
				attrib = new XAttributeDiscreteImpl(key, (Integer) value);
			else if (value instanceof Float || value instanceof Double)
				attrib = new XAttributeContinuousImpl(key, (Double) value);
			else if (value instanceof Date)
				attrib = new XAttributeTimestampImpl(key, (Date) value);
			else
				attrib = new XAttributeLiteralImpl(key, value.toString());
			newEvent.getAttributes().put(key, attrib);
		}
		return newEvent;
	}
	
	public void saveInstances(File file, Instances instances) throws IOException {
		ArffSaver saver = new ArffSaver();
		saver.setInstances(instances);
		saver.setFile(file);
		saver.writeBatch();
	}
	
	/**
	 * 
	 * @param file
	 * @param conf
	 * @throws IOException
	 */
	public void savePrefixes(File file, Configuration conf) throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(file));
	    writer.write("LOG MOVES prefixes\n\n"); 
		for(String activity:conf.getPc(false).keySet()) {
	    	ArrayList<PrefixFrequencyPair> prefixes=conf.getPc(false).getPrefixes(activity);
	    	for(int i=0;i<prefixes.size();i++)
	    		writer.write(activity+" ("+i+") :"+prefixes.get(i).getPrefix().toString()+"\n");
	    }
		writer.write("MODEL MOVES prefixes\n\n");
		for(String activity:conf.getPc(true).keySet()) {
	    	ArrayList<PrefixFrequencyPair> prefixes=conf.getPc(true).getPrefixes(activity);
	    	for(int i=0;i<prefixes.size();i++)
	    		writer.write(activity+" ("+i+") :"+prefixes.get(i).getPrefix().toString()+"\n");
	    }
		
	    writer.close();
		
	}
	
	public void saveAprioriOutput(File file, String output,RuleSet ruleset) throws IOException {
		FileWriter writer = new FileWriter(file);
		
		writer.write("Class:"+ruleset.getClassValue()+" Rule:\n");
		for(Rule r:ruleset.getList()) {
			writer.write("\t"+r.getBody()+r.getType()+r.getValue()+"\n");
		}
		writer.write("------------------------------------------");
		
		Scanner scanner = new Scanner(output);
		while (scanner.hasNextLine()) {
		  writer.write(scanner.nextLine()+"\n");
		}
		scanner.close();
		writer.close();
	}
	

	private void save_AprioriOutputFile(RuleSet ruleset, ArrayList<RuleSet> newRules) throws IOException{
		FileWriter writer = new FileWriter(new File(FILE_SELECTED_APRIORI_RULES),true);
		
		writer.write("Class:"+ruleset.getClassValue()+" Rule:\n");
		for(Rule r:ruleset.getList()) {
			writer.write("\t"+r.getBody()+r.getType()+r.getValue()+"\n");
		}
		for(RuleSet rs:newRules) {
			writer.write("\nNew rules from apriori:\n");
			for(Rule r:rs.getList())
				writer.write("\t"+r.getBody()+r.getType()+r.getValue()+"\n");
		}
		writer.write("\n");
		writer.close();
}

	private void resetAprioriOutputFile() throws IOException{
		PrintWriter writer = new PrintWriter(new File(FILE_SELECTED_APRIORI_RULES));
		writer.print("");
		writer.close();
}


	
	/**
	 * Create an Instances object with all instances for all clusters
	 * @param conf
	 * @param rr
	 * @return
	 */
	public Instances extractAllInstances(Configuration conf,  ResultReplay rr) {
		ArrayList<Attribute> listAttribute=createAttributeList(getTypes(conf), new HashMap<String, Set<String>>() , conf.getIndicatorValues().toArray());
		Instances output=new Instances("relation",listAttribute,100);
		int i=-1;
		for(XTrace trace:rr.getAlignedLog()) {
			i++;
			String traceid=XConceptExtension.instance().extractName(trace);
			Instance inst=new DenseInstance(listAttribute.size());
			int traceClass=Integer.parseInt(getIndicatorValue(trace, conf).toString());
			for(Attribute attr:listAttribute) {
				int num=0;
				//for class attribute
				if(attr.toString().contains("CLASS")) 
					//continue;
					num=traceClass+1;
				//for attributes with variants
				else if(attr.toString().contains("("))
					num=getNumMovesinTracesForVariableWithPrefixes(traceid, attr.name(), conf);
				//for generic attributes
				else	
					num=getNumMovesinTraces(traceid, attr.name(), conf);
				inst.setValue(attr, num);
				//add the instance to the general list in the Configuration class
				
				}
			output.add(inst);
		}
		return output;
    }
}